//
//  Bluetooth4Obd.m
//  HA7 logger mini04
//  Copyright(C)2012-2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.04.30 v0.0  First cording
//    2013.05.04 v0.4  Commit
//
#import <CoreBluetooth/CoreBluetooth.h>
#import "Bluetooth4Obd.h"

@interface Bluetooth4Obd () <CBCentralManagerDelegate, CBPeripheralDelegate>
@property (strong, nonatomic) CBCentralManager      *centralManager;
@property (strong, nonatomic) CBPeripheral          *discoveredPeripheral;
@property (strong, nonatomic) NSTimer               *updateTimer;
@end


@implementation Bluetooth4Obd


- (id)init
{
    self = [super init];
    if (self) {
        self.centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
#ifdef READ_INTERVAL
        self.updateTimer = [NSTimer scheduledTimerWithTimeInterval:READ_INTERVAL
                                                            target:self
                                                          selector:@selector(polingObdData:)
                                                          userInfo:nil
                                                           repeats:YES];
#endif
    }
    return self;
}


#pragma mark - Bluetooth-LE central


- (void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    if (central.state != CBCentralManagerStatePoweredOn) {
        return;
    }
    [self scan];
}


- (void)scan
{
    [self.centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:OBD_SERVICE_UUID]] options:nil];
//options:@{ CBCentralManagerScanOptionAllowDuplicatesKey : @YES }];
    NSLog(@"Scanning...");
}


- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    NSLog(@"Discovered %@ at %@db", peripheral.name, RSSI);
    if (self.discoveredPeripheral != peripheral) {
        self.discoveredPeripheral = peripheral;
        NSLog(@"Connecting to peripheral %@", peripheral);
        [self.centralManager connectPeripheral:peripheral options:nil];
    }
}


- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Failed to connect to %@. (%@)", peripheral, [error localizedDescription]);
    [self cleanup];
}


- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"Peripheral Connected");
    [self.centralManager stopScan];
    peripheral.delegate = self;
    [peripheral discoverServices:@[[CBUUID UUIDWithString:OBD_SERVICE_UUID]]];
    self.valid = 1;
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering services: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBService *service in peripheral.services) {
        [peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID1]] forService:service];
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
{
    if (error) {
        NSLog(@"Error discovering characteristics: %@", [error localizedDescription]);
        [self cleanup];
        return;
    }
    for (CBCharacteristic *characteristic in service.characteristics) {
#ifdef READ_INTERVAL
        [peripheral readValueForCharacteristic:characteristic];
#else
        [peripheral setNotifyValue:YES forCharacteristic:characteristic];
#endif
    }
}


- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    int temp;
    
    if (error) {
        NSLog(@"Error discovering characteristics: %@", [error localizedDescription]);
        return;
    }
    temp = *((short *)[characteristic.value bytes]);    // Byte order is Little endian
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID1]]) {
        self.ect = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID2]]) {
        self.shrtft1 = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID3]]) {
        self.longft1 = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID4]]) {
        self.map = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID5]]) {
        self.rpm = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID6]]) {
        self.vss = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID7]]) {
        self.sparkadv = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID8]]) {
        self.iat = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID9]]) {
        self.tp = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID10]]) {
        self.o2s11 = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID11]]) {
        self.shrtft11 = temp;
    }
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:OBD_CHARACTERISTIC_UUID12]]) {
        self.vpwr = temp;
    }
}


- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Peripheral Disconnected");
    self.discoveredPeripheral = nil;
    [self scan];
    self.valid = 0;
}


- (void)cleanup
{
    if (!self.discoveredPeripheral.isConnected) {
        return;
    }
#ifndef READ_INTERVAL
    if (self.discoveredPeripheral.services != nil) {
        for (CBService *service in self.discoveredPeripheral.services) {
            if (service.characteristics != nil) {
                for (CBCharacteristic *characteristic in service.characteristics) {
                    if (characteristic.isNotifying) {
                        [self.discoveredPeripheral setNotifyValue:NO forCharacteristic:characteristic];
                    }
                }
            }
        }
    }
#endif
    [self.centralManager cancelPeripheralConnection:self.discoveredPeripheral];
}

#ifdef READ_INTERVAL
- (IBAction)polingObdData:(id)sender {
    if (self.discoveredPeripheral.services != nil) {
        for (CBService *service in self.discoveredPeripheral.services) {
            if (service.characteristics != nil) {
                for (CBCharacteristic *characteristic in service.characteristics) {
                    [self.discoveredPeripheral readValueForCharacteristic:characteristic];
                }
            }
        }
    }
}
#endif

@end
