//
//  ObdDataLogger.m
//  HA7 logger mini04
//  Copyright(C)2012-2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.04.30 v0.0  First cording
//    2013.05.04 v0.4  Commit
//
#import <CoreLocation/CoreLocation.h>
#import "ObdDataLogger.h"
#import "Bluetooth4Obd.h"

@interface ObdDataLogger ()
@property (strong, nonatomic) Bluetooth4Obd         *obd;
@property (strong, nonatomic) NSTimer               *updateTimer;
@property (strong, nonatomic) CLLocationManager     *locationManager;
@property (nonatomic)         FILE                  *fp;
@end

@implementation ObdDataLogger


- (id)init
{
    self = [super init];
    if (self) {
        self.obd = [[Bluetooth4Obd alloc] init];
        
        self.updateTimer = [NSTimer scheduledTimerWithTimeInterval:INTVL
                                                            target:self
                                                          selector:@selector(logOne:)
                                                          userInfo:nil
                                                           repeats:YES];
        
        self.locationManager = [[CLLocationManager alloc] init];
//      self.locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters;
        self.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
//      self.locationManager.distanceFilter  = 10;
        self.locationManager.distanceFilter  = 1;
    }
    NSLog(@"ObdDataLogger init");
    return self;
}


#pragma mark - Background task


- (IBAction)logOne:(id)sender
{

    // Connected
    self.valid = self.obd.valid;
    if (self.obd.valid) {
        
        if (self.nowLoc == nil) {
            [self.locationManager startUpdatingLocation];    // GPS on
        }

		// Check point
        self.ect      = self.obd.ect;
        self.shrtft1  = self.obd.shrtft1;
        self.longft1  = self.obd.longft1;
        self.map      = self.obd.map;
        self.rpm      = self.obd.rpm;
        self.vss      = self.obd.vss;
        self.sparkadv = self.obd.sparkadv;
        self.iat      = self.obd.iat;
        self.tp       = self.obd.tp;
        self.o2s11    = self.obd.o2s11;
        self.shrtft11 = self.obd.shrtft11;
        self.vpwr     = self.obd.vpwr;
        self.nowLoc   = self.locationManager.location;

		// Calcurate
		self.atm  = self.map * 1000.0 / 101325.0;           // Air pressure [atm]
		self.r    = (1.293 * self.atm) / (1 + 0.00367 * self.iat);  // Air density [kg/m^3]
		self.v    = self.rpm / 60.0 / E_CYC * C_VOL;        // Air volume [m^3/sec]
		self.af   = self.r * self.v * 1000.0;               // Air flow on intake manifold [g/sec]
        
		self.ff   = self.af / 14.7
                  * (self.shrtft11 / 100.0 + 1.0);          // Fuel flow on intake manifold [g/sec]
		
		self.dist = (self.vss / (60.0 * 60.0) * INTVL);	    // Interval distance [km]
		self.fuse = (self.ff / 0.783 / 1000.0 * INTVL);     // Interval used fuel [l]
        
		// Estimate
		self.dist_s  += self.dist;                          //
		self.dist_d  += self.dist;                          //
		self.fuse_s  += self.fuse;                          //
		self.fuse_d  += self.fuse;                          //
		self.runtm   += INTVL;								//
		self.runtm_d += INTVL;								//
        if (self.dist > 0.0001) {                           //
            self.movtm   += INTVL;							//
            self.movtm_d += INTVL;							//
        }
        self.remainFuel -= self.fuse;                       //
        
		// Fuel efficient
		if (self.dist < 0.0001) {							// 10[cm]
			self.eff = 0.0;									//
		} else if (self.fuse > 0.00001) {					// 0.01[ml]
			self.eff = self.dist / self.fuse;               // [km/l]
		} else {                                            //
			self.eff = 99.9;								//
		}                                                   //
		if (self.dist_s < 0.001) {							// 1[m]
			self.eff_s = 0;									//
		} else if (self.fuse_s > 0.001) {					// 1[ml]
			self.eff_s = self.dist_s / self.fuse_s;         //
		} else {                                            //
			self.eff_s = 99.9;								//
		}                                                   //
		if (self.dist_d < 0.001) {							// 1[m]
			self.eff_d = 0;									//
		} else if (self.fuse_d > 0.001) {					// 1[ml]
			self.eff_d = self.dist_d / self.fuse_d;         //
		} else {                                            //
			self.eff_d = 99.9;								//
		}                                                   //
        
        // Logging
        {
            static int  yy = 0;
            static int  mm = 0;
            static int  dd = 0;
			time_t      t;
			struct tm   *tm;
            int         y;
            int         m;
            int         d;
            
			time(&t);
			tm = localtime(&t);
            y = tm->tm_year + 1900;
            m = tm->tm_mon + 1;
            d = tm->tm_mday;
            if ((y != yy) || (m != mm) || (d != dd) || (self.fp == NULL)) {
                if (self.fp != NULL) {
                    yy = y;
                    mm = m;
                    dd = d;
                    self.dist_d  = 0;
                    self.fuse_d  = 0;
                    self.runtm_d = 0;
                    self.movtm_d = 0;

                    fclose(self.fp);
                }

                NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
                NSString *path1 = [[NSString alloc] initWithFormat:@"%@/%d-%02d-%02d.csv",[paths objectAtIndex:0],y,m,d];
                const char *fname = [path1 UTF8String];
                if ((self.fp = fopen(fname,"a")) != NULL) {
                    fprintf(self.fp, "\"Date_Time\",\"ECT[~C]\",\"SHRTFT1[%%]\",\"LONGFT1[%%]\",\"MAP[kPa]\",\"RPM[r/m]\",\"VSS[km/h]\",\"SPARKADV[deg]\",\"IAT[~C]\",\"TP[%%]\",\"O2S11[mV]\",\"SHRTFT11[%%]\",\"VPWR[V]\",\"DIST[km]\",\"FUSE[l]\",\"EFF[km/l]\",\"DIST_SUM[km]\",\"FUSE_SUM[l]\",\"EFF_AVE[km/l]\",\"RUNTM[s]\",\"LATITUDE[deg]\",\"LONGITUDE[deg]\",\"ALTITUDE[m]\"\n");
                } else {
                    NSLog(@"fopen error=%d",errno);
                }
            }
            if (self.fp != NULL) {
                fprintf(self.fp, "\"%d-%02d-%02d %02d:%02d:%02d\",", y, m, d, tm->tm_hour, tm->tm_min, tm->tm_sec);
                fprintf(self.fp, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%.2f,%.4f,%.4f,%.1f,%.2f,%.2f,%.1f,%ld,%+.6f,%+.6f,%+.6f\n",
                        self.obd.ect, self.obd.shrtft1, self.obd.longft1, self.obd.map, self.obd.rpm, self.obd.vss, self.obd.sparkadv, self.obd.iat, self.obd.tp, self.obd.o2s11, self.obd.shrtft11, self.obd.vpwr/1000.0,
                        self.dist, self.fuse, self.eff,
                        self.dist_s, self.fuse_s, self.eff_s, self.runtm,
                        self.nowLoc.coordinate.latitude,
                        self.nowLoc.coordinate.longitude,
                        self.nowLoc.altitude);
            }
        }
    
    // Disconnected
    } else {
        if (self.fp != NULL) {
            fclose(self.fp);
            self.fp = NULL;
        }
        if (self.nowLoc != nil) {
            [self.locationManager stopUpdatingLocation];    // GPS off
            self.nowLoc = nil;
        }
    }
}



@end
